/*
  gkrellscore plugin to GKrellM- shows sports scores
  Copyright (C) 2001 paul cannon

  This program is free software; you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation; either version 2
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
      02111-1307, USA.

  To contact the author try:
  <paul@cannon.cs.usu.edu>
*/

#include <gkrellm/gkrellm.h>
#include <errno.h>

#define PLUGIN_NAME "GKrellScore"
#define PLUGIN_DESC "GKrellM Sports Scores ticker"
#define PLUGIN_VER "0.0.2"
#define PLUGIN_STYLE PLUGIN_NAME
#define PLUGIN_KEYWORD PLUGIN_NAME
#define PLUGIN_URL "http://ssl.usu.edu/paul/gkrellscore/"

static char *gkscore_info_text[] =
{
  "<b>" PLUGIN_NAME " " PLUGIN_VER "\n\n",

  "This plugin keeps you up-to-date on sports scores,\n",
  "letting you stay at your computer and write geeky\n",
  "programs while keeping tabs on your favorite teams!\n\n",

  "This is an early version, and doesn't do much besides\n",
  "download scores from the internet and display them in\n",
  "a little panel. The scores come from sports.excite.com,\n",
  "and you can click on the score panel to launch your\n",
  "browser and bring up a page specifically for the current\n",
  "game.\n\n",

  "Start by selecting the league of your team. After that,\n",
  "you can click on the \"Browse Teams\" button to launch\n",
  "your browser (configurable) to show the page for the\n",
  "league you've selected. Find the specific page for your\n",
  "team, and look at the last part of its URL. That is the\n",
  "code for your team. For example, the NY Jets page is at\n",
  "http://sports.excite.com/nfl/nyj/, so the team code is\n",
  "nyj. Once the league and team code are known, the plugin\n",
  "will periodically get the scores for any current game\n",
  "in which that team is playing, or if none, the last game\n",
  "in which it played. If a game is to be played later in\n",
  "the day, then the starting time will be displayed.\n\n",

  "The scores are displayed as \"",
  "<b>hom (h)-(v) vis",
  "\", where\n",
  "hom is the home team's code, vis is the visiting team's\n",
  "code, and (h) and (v) are their respective scores. More\n",
  "in-depth information is available by looking at the\n",
  "tooltip. Just hold your mouse cursor over the score box\n",
  "for a few seconds.\n\n",
  
  "<b>For more info, see " PLUGIN_URL
};

static char *gkscore_about_text = _(
  PLUGIN_NAME " " PLUGIN_VER
  "\n" PLUGIN_DESC
  "\n\nCopyright (C) 2001 paul cannon\n"
  "paul@cannon.cs.usu.edu\n"
  "space software lab/utah state university\n\n"
  PLUGIN_NAME " comes with ABSOLUTELY NO WARRANTY;\n"
  "see the file COPYING for details.\n\n"
  PLUGIN_URL );

static char *leagues[] = {
  "nhl", "NHL - National Hockey League",
  "ihl", "IHL - International Hockey League",
  "ncaah", "NCAA Hockey",
  "ahl", "AHL - American Hockey League",
  "ehcl", "East Coast Hockey League",
  
  "mlb", "MLB - Major League Baseball",
  "ncaad", "NCAA Baseball",
  
  "nba", "NBA - National Basketball Association",
  "wnba", "WNBA - Women's National Basketball Association",
  "ncaab", "NCAA Men's Basketball",
  "ncaaw", "NCAA Women's Basketball",
  "cba", "Canadian Basketball Association",
  
  "nfl", "NFL - National Football League",
  "ncaaf", "NCAA Football",
  "cfl", "Canadian Football League",
  "arena", "Arena Football",
  "wlaf", "NFL Europe",
  "xfl", "XFL",
  
  "mls", "MLS - Major League Soccer",
  "bund", "Bundesliga Soccer",
  "dutch", "Dutch Soccer",
  "premr", "Premier Soccer (England)",
  "span", "Spanish Primera Soccer",
  "serie", "Serie A Soccer",
  
  NULL
};

#define TIP_INFO        _("GKrellScore info from excite.com")
#define SPORTS_URL      "sports.excite.com"
#define SCORECMD        "gkrellscore.excite %s %s"
#define DEFAULT_BROWSER "netscape" /* konqueror is better */
#define MAX_BUFLEN      256

static int debugging = 0;

static Panel *panel = NULL;
static Decal *decal = NULL;
static gint style_id;

static gchar *score_text = NULL;

static int gkscore_period = 1;
static gchar *new_league = NULL;
static gchar *gkscore_league = NULL;
static gchar *gkscore_team = NULL;
static gchar *gkscore_browser = NULL;

static GtkWidget *period_spinner;
static GtkWidget *league_box;
static GtkWidget *team_box;
static GtkWidget *browser_box;
static GtkTooltips *summary;

#ifdef GKSCORE_MYHOME
/* I don't believe I'll implement this feature anymore, since I don't
   plan to use the H %d-%d V format */
static gboolean gkscore_myhome = FALSE;
static GtkWidget *myhomecheck;
#endif

static FILE *cmd_pipe = NULL;

static gchar *hometeam = NULL;
static gchar *homescore = NULL;
static gchar *visitorteam = NULL;
static gchar *visitorscore = NULL;
static gchar *gameperiod = NULL;
static gchar *gamedate = NULL;
static gchar *gamerole = NULL;
static gchar *gameurl = NULL;

static gchar *alloc_fgets (FILE *f)
{
  gchar buf[MAX_BUFLEN];
  int len;

  if (fgets (buf, MAX_BUFLEN, f) == NULL)
    return NULL;
  
  len = strlen (buf);
  if (len && buf[len - 1] == '\n')
    buf[len - 1] = '\0';

  return g_strdup (buf);
}

static void gkscore_set_text (gchar *text)
{
  int width;
  
  if (text == NULL)
    return;

  g_free (score_text);
  score_text = g_strdup (text);

  if (panel == NULL)
    return;

  width = gdk_string_width (decal->text_style.font, score_text);
  decal->x_off = (gkrellm_chart_width () - width) / 2;
  gkrellm_draw_decal_text (panel, decal, score_text, -1);
  gkrellm_draw_layers (panel);
}

static void gkscore_update_data ()
{
  gchar *newtext, *tooltip;
  gchar *homecode = NULL, *viscode = NULL;
  char *paren;

  if (gameurl == NULL) /* it's safe to say if the others are, this one is too */
    return;

  if (homescore[0] && visitorscore[0])
  {
    if ((paren = strchr (hometeam, '(')) != NULL)
    {
      homecode = g_strdup (++paren);
      if ((paren = strchr (homecode, ')')) != NULL)
        *paren = '\0';
    }
    else
      homecode = g_strdup ("H");

    if ((paren = strchr (visitorteam, '(')) != NULL)
    {
      viscode = g_strdup (++paren);
      if ((paren = strchr (viscode, ')')) != NULL)
        *paren = '\0';
    }
    else
      viscode = g_strdup ("V");

    newtext = g_strdup_printf ("%s %s-%s %s",
                               homecode, homescore,
                               visitorscore, viscode);
  }
  else if (gameperiod[0])
    newtext = g_strdup (gameperiod);
  else
    newtext = g_strdup (_("No score."));
  
  gkscore_set_text (newtext);
  g_free (newtext);
  
  tooltip = g_strdup_printf ("%s\n%s %s\n%s %s\n%s - %s", TIP_INFO,
                             hometeam, homescore,
                             visitorteam, visitorscore,
                             gameperiod, gamedate);
  gtk_tooltips_set_tip (summary, panel->drawing_area, tooltip, NULL);
  g_free (tooltip);
}

static void start_score_update ()
{
  char *cmd;

  if (cmd_pipe) /* already open */
    return;

  if (gkscore_league == NULL || gkscore_team == NULL)
  {
    gkscore_set_text (_("No config."));
    return;
  }
  if (!strncmp (score_text, _("No "), 3))
    gkscore_set_text (_("updating.."));

  cmd = g_strdup_printf (SCORECMD, gkscore_league, gkscore_team);
  cmd_pipe = popen (cmd, "r");
  g_free (cmd);

  if (cmd_pipe)
    fcntl (fileno (cmd_pipe), F_SETFL, O_NONBLOCK);
  else if (debugging)
    fprintf (stderr, "%s: couldn't popen command pipe!\n", PLUGIN_NAME);
}

static int get_pipe_results ()
{
  gchar c;

  if (cmd_pipe == NULL)
    return 0;

  if ((c = fgetc (cmd_pipe)) == EOF)
  {
    /* if we get EAGAIN, wait some more */
    if (ferror (cmd_pipe) && errno == EAGAIN)
    {
      if (debugging > 1)
        fprintf (stderr, "%s: no I/O ready at pipe.\n", PLUGIN_NAME);
      return 0;
    }

    /* if we reach here something has gone wrong with the pipe.
       return error code */
    if (debugging)
      fprintf (stderr, "%s: the pipe broke..\n", PLUGIN_NAME);
    pclose (cmd_pipe);
    cmd_pipe = NULL;
    return -1;
  }

  /* if we get here the pipe is ready! */

  if (debugging > 1)
    fprintf (stderr, "%s: Got info in pipe!\n", PLUGIN_NAME);

  ungetc (c, cmd_pipe);

  g_free (hometeam);
  hometeam = alloc_fgets (cmd_pipe);
  g_free (homescore);
  homescore = alloc_fgets (cmd_pipe);
  g_free (visitorteam);
  visitorteam = alloc_fgets (cmd_pipe);
  g_free (visitorscore);
  visitorscore = alloc_fgets (cmd_pipe);
  g_free (gameperiod);
  gameperiod = alloc_fgets (cmd_pipe);
  g_free (gamedate);
  gamedate = alloc_fgets (cmd_pipe);
  g_free (gamerole);
  gamerole = alloc_fgets (cmd_pipe);
  g_free (gameurl);
  gameurl = alloc_fgets (cmd_pipe);

  if (debugging > 1)
  {
    fprintf (stderr, "%s %s\n",
             homescore, visitorscore);
  }

  pclose (cmd_pipe);
  cmd_pipe = NULL;

  gkscore_update_data ();

  return 1;
}

static void gkscore_update_plugin ()
{
  static int count = 0;

  if (cmd_pipe && GK.two_second_tick)
  {
    get_pipe_results ();
    count = 0;
  }

  if (GK.minute_tick && (count = (count + 1) % gkscore_period) == 0)
    start_score_update ();
}

static gint gkscore_panel_expose_event (GtkWidget *widget, GdkEventExpose *ev)
{
  gdk_draw_pixmap (widget->window,
      widget->style->fg_gc[GTK_WIDGET_STATE (widget)],
      panel->pixmap, ev->area.x, ev->area.y, ev->area.x, ev->area.y,
      ev->area.width, ev->area.height);
  return FALSE;
}

static gint gkscore_click (GtkWidget *w, GdkEventButton *ev)
{
  gchar *browsecmd = NULL;

  switch (ev->button)
  {
  case 1:
    if (gameurl)
      browsecmd = g_strdup_printf ("%s %s > /dev/null 2>&1 &",
                                   gkscore_browser, gameurl);
    else if (gkscore_league && gkscore_team)
      browsecmd = g_strdup_printf ("%s http://%s/%s/%s/ > /dev/null 2>&1 &",
                                   gkscore_browser,
                                   SPORTS_URL, gkscore_league, gkscore_team);
    if (browsecmd)
    {
      system (browsecmd);
      g_free (browsecmd);
    }
    break;
  case 3:
    gkscore_set_text (_("updating.."));
    start_score_update ();
    break;
  }
  return FALSE;
}

static void gkscore_create_plugin (GtkWidget *vbox, gint first_create)
{
  Style *style;
  TextStyle *ts, *ts_alt;
  int width;

  if (first_create)
  {
    start_score_update ();
    panel = gkrellm_panel_new0 ();
  }
  else
    gkrellm_destroy_decal_list (panel);

  style = gkrellm_meter_style (style_id);

  ts = gkrellm_meter_textstyle (style_id);
  ts_alt = gkrellm_meter_alt_textstyle (style_id);
  panel->textstyle = ts;

  decal = gkrellm_create_decal_text (panel, "0", ts_alt,
                                     style, -1, -1, -1);
  
  gkrellm_configure_panel (panel, NULL, style);
  panel->label->h_panel += 2;   /* Some bottom margin */
  
  gkrellm_create_panel (vbox, panel, gkrellm_bg_meter_image (style_id));
  gkrellm_monitor_height_adjust (panel->h);

  width = gdk_string_width (decal->text_style.font, score_text);
  decal->x_off = (gkrellm_chart_width () - width) / 2;
  gkrellm_draw_decal_text (panel, decal, score_text, -1);

  gkrellm_draw_layers (panel);

  if (first_create)
  {
    gtk_signal_connect (GTK_OBJECT (panel->drawing_area), "expose_event",
                        GTK_SIGNAL_FUNC (gkscore_panel_expose_event), NULL);
    gtk_signal_connect (GTK_OBJECT (panel->drawing_area), "button_press_event",
                        GTK_SIGNAL_FUNC (gkscore_click), NULL);

    summary = gtk_tooltips_new ();
    gtk_tooltips_set_tip (summary, panel->drawing_area, PLUGIN_NAME, NULL);
    gtk_tooltips_set_delay (summary, 1000);
  }
}

static void gkscore_save_config (FILE *f)
{
  if (gkscore_league)
    fprintf (f, "%s league %s\n", PLUGIN_KEYWORD, gkscore_league);
  if (gkscore_team)
    fprintf (f, "%s team %s\n", PLUGIN_KEYWORD, gkscore_team);
  if (gkscore_browser)
    fprintf (f, "%s browser %s\n", PLUGIN_KEYWORD, gkscore_browser);
  fprintf (f, "%s period %d\n", PLUGIN_KEYWORD, gkscore_period);
#ifdef GKSCORE_MYHOME
  fprintf (f, "%s myhome %d\n", PLUGIN_KEYWORD, gkscore_myhome);
#endif
}

static void gkscore_load_config (gchar *arg)
{
  gchar *config_item, *value;

  config_item = strtok (arg, " \n");
  if (!config_item)
    return;
  value = strtok (NULL, "\n");
  if (!value)
    return;

  if (!strcmp (config_item, "league"))
  {
    g_free (gkscore_league);
    gkscore_league = g_strdup (value);
  }
  else if (!strcmp (config_item, "team"))
  {
    g_free (gkscore_team);
    gkscore_team = g_strdup (value);
  }
  else if (!strcmp (config_item, "browser"))
  {
    g_free (gkscore_browser);
    gkscore_browser = g_strdup (value);
  }
  else if (!strcmp (config_item, "period"))
  {
    gkscore_period = atoi (value);
    gkscore_period = MAX (gkscore_period, 1);
  }
#ifdef GKSCORE_MYHOME
  else if (!strcmp (config_item, "myhome"))
    gkscore_myhome = CLAMP (atoi (value), 0, 1);
#endif
}

static gint pick_menuitem (gpointer gnum)
{
  int num;

  num = GPOINTER_TO_INT (gnum);
  gtk_option_menu_set_history (GTK_OPTION_MENU (league_box), num);
  g_free (new_league);
  new_league = g_strdup (leagues[num * 2]);
  if (gkscore_league && strcmp (new_league, gkscore_league))
    gtk_entry_set_text (GTK_ENTRY (team_box), "");

  return FALSE;
}

static void browse_teams ()
{
  gchar *cmd, *url, *league;

  if (new_league == NULL)
    league = gkscore_league;
  else
    league = new_league;

  if (league == NULL)
    return;

  url = g_strdup_printf ("http://%s/%s/", SPORTS_URL, league);
  cmd = g_strdup_printf ("%s %s > /dev/null 2>&1 &", gkscore_browser, url);
  g_free (url);
  system (cmd);
  g_free (cmd);
}

static void gkscore_create_tabs (GtkWidget *tab_vbox)
{
  GtkWidget *tabs;
  GtkWidget *vbox, *hbox;
  GtkWidget *scrolled;
  GtkWidget *text, *label, *leaguemenu, *menuitem, *button;
  GtkWidget *about, *aboutlabel;
  int i, active = 0;

  tabs = gtk_notebook_new ();
  gtk_notebook_set_tab_pos (GTK_NOTEBOOK (tabs), GTK_POS_TOP);
  gtk_box_pack_start (GTK_BOX (tab_vbox), tabs, TRUE, TRUE, 0);

  /* Setup Tab */
  vbox = gkrellm_create_tab (tabs, _("Options"));

  label = gtk_label_new (_("League:"));
  league_box = gtk_option_menu_new ();
  leaguemenu = gtk_menu_new ();
  for (i = 0; leagues[i * 2]; i++)
  {
    menuitem = gtk_menu_item_new_with_label (leagues[i * 2 + 1]);
    if (gkscore_league && !strcmp (leagues[i * 2], gkscore_league))
      active = i;
    gtk_signal_connect_object (GTK_OBJECT (menuitem), "activate",
                       GTK_SIGNAL_FUNC (pick_menuitem), GINT_TO_POINTER (i));
    gtk_menu_append (GTK_MENU (leaguemenu), menuitem);
  }
  gtk_option_menu_set_menu (GTK_OPTION_MENU (league_box), leaguemenu);
  gtk_option_menu_set_history (GTK_OPTION_MENU (league_box), active);
  g_free (new_league);
  new_league = g_strdup (leagues[active * 2]);

  hbox = gtk_hbox_new (FALSE, 10);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), league_box, TRUE, TRUE, 0);
  button = gtk_button_new_with_label (_("Browse teams"));
  gtk_signal_connect (GTK_OBJECT (button), "clicked",
                      GTK_SIGNAL_FUNC (browse_teams), NULL);
  gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, FALSE, 0);
  
  hbox = gtk_hbox_new (FALSE, 10);
  label = gtk_label_new (_("Team:"));
  team_box = gtk_entry_new ();
  if (gkscore_team)
    gtk_entry_set_text (GTK_ENTRY (team_box), gkscore_team);
  gtk_entry_set_editable (GTK_ENTRY (team_box), TRUE);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), team_box, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, FALSE, 0);
 
  gkrellm_spin_button (vbox, &period_spinner,
                       (gfloat) gkscore_period,
                       1.0, 500.0, 1.0, 10.0, 0, 60, NULL, NULL,
                       FALSE, _("Minutes per update"));

  hbox = gtk_hbox_new (FALSE, 10);
  label = gtk_label_new (_("Browser:"));
  browser_box = gtk_entry_new ();
  gtk_entry_set_text (GTK_ENTRY (browser_box), gkscore_browser);
  gtk_entry_set_editable (GTK_ENTRY (browser_box), TRUE);
  gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
  gtk_box_pack_start (GTK_BOX (hbox), browser_box, TRUE, TRUE, 0);
  gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, FALSE, 0);

#ifdef GKSCORE_MYHOME
  gkrellm_check_button (vbox, &myhomecheck, gkscore_myhome, FALSE, 5,
             _("Show my team always on the left (instead of the home team)"));
#endif

  /* Info tab */

  vbox = gkrellm_create_tab (tabs, _("Info"));
  scrolled = gtk_scrolled_window_new (NULL, NULL);
  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
      GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
  gtk_box_pack_start (GTK_BOX (vbox), scrolled, TRUE, TRUE, 0);
  text = gtk_text_new (NULL, NULL);
  gkrellm_add_info_text (text, gkscore_info_text,
      sizeof (gkscore_info_text) / sizeof (gchar *));
  gtk_text_set_editable (GTK_TEXT (text), FALSE);
  gtk_container_add (GTK_CONTAINER (scrolled), text);

  /* About tab */

  about = gtk_label_new (gkscore_about_text);
  aboutlabel = gtk_label_new (_("About"));
  gtk_notebook_append_page (GTK_NOTEBOOK (tabs), about, aboutlabel);
}

static void gkscore_apply_config ()
{
  gchar *newteam;
  gint diff;

  newteam = gtk_editable_get_chars (GTK_EDITABLE (team_box), 0, -1);

  if (gkscore_league == NULL || gkscore_team == NULL)
    diff = 1;
  else if (strcmp (newteam, gkscore_team))
    diff = 1;
  else if (new_league == NULL)
    diff = 0;
  else
    diff = strcmp (new_league, gkscore_league);

  if (new_league)
  {
    g_free (gkscore_league);
    gkscore_league = g_strdup (new_league);
  }
  g_free (gkscore_team);
  gkscore_team = newteam;
  
  g_free (gkscore_browser);
  gkscore_browser = g_strstrip (gtk_editable_get_chars (
                                          GTK_EDITABLE (browser_box), 0, -1));

  gkscore_period = gtk_spin_button_get_value_as_int
                                   (GTK_SPIN_BUTTON (period_spinner));

#ifdef GKSCORE_MYHOME
  gkscore_myhome = GTK_TOGGLE_BUTTON (myhomecheck)->active;
#endif

  if (diff)
  {
    gkscore_set_text (_("updating.."));
    start_score_update ();
  }
  else
    gkscore_update_data ();
}

static Monitor gkscore_plugin_mon =
{
  PLUGIN_NAME,            /* Name, for config tab.                    */
  0,                      /* Id,  0 if a plugin                       */
  gkscore_create_plugin,  /* The create function                      */
  gkscore_update_plugin,  /* The update function                      */
  gkscore_create_tabs,    /* The config tab create function           */
  gkscore_apply_config,   /* Apply the config function                */

  gkscore_save_config,    /* Save user config                         */
  gkscore_load_config,    /* Load user config                         */
  PLUGIN_KEYWORD,         /* config keyword                           */

  NULL,                   /* Undefined 2                              */
  NULL,                   /* Undefined 1                              */
  NULL,                   /* private                                  */

  MON_MAIL,               /* Insert plugin before this monitor        */

  NULL,                   /* Handle if a plugin, filled in by GKrellM */
  NULL                    /* Path if a plugin, filled in by GKrellM   */
};

Monitor *init_plugin ()
{
  style_id = gkrellm_add_meter_style (&gkscore_plugin_mon, PLUGIN_NAME);
  score_text = g_strdup (_("updating.."));
  gkscore_browser = g_strdup (DEFAULT_BROWSER);

  return &gkscore_plugin_mon;
}
